"""Autogenerated API"""
from argus_api import session
import logging
from argus_cli.plugin import register_command
from argus_plugins import argus_cli_module
log = logging.getLogger(__name__)


@register_command(
    extending=("eventfilters", "v2", "analysis"),
    module=argus_cli_module
)
def add_analysis_filter(
    name: str = None,
    description: str = None,
    customer: str = None,
    preStatementCode1: str = None,
    preStatementCode2: str = None,
    statementCode: str = None,
    triggerCode: str = None,
    actionName: str = None,
    labels: str = None,
    comment: str = None,
    validFrom: int = None,
    validTo: int = None,
    index: int = 5,
    defaultAggregate: bool = None,
    skipNewEvent: bool = None,
    enabled: bool = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    """Adds an analysis filter (DEV)
    
    :param str name: The name of the filter 
    :param str description: A description of the filter 
    :param str customer: The shortname or ID of the customer the filter belongs to. If customer is not set\, this will create a global filter 
    :param str preStatementCode1: Optional esper code which does a pre selection before the statement code is run 
    :param str preStatementCode2: Optional esper code to do further pre selection on the result of preStatementCode1\, and applied before the statementCode 
    :param str statementCode: Esper code to select which events the filter will be applied to 
    :param str triggerCode: Groovy code which is run on the selected events 
    :param str actionName: The name of the signature which should be attached to the created event 
    :param list labels: A set of labels which can be used to categorize the filter 
    :param str comment: A comment describing why the filter was created 
    :param int validFrom: When the filter starts being applied. Timestamp can be milliseconds since epoch\, ISO8601 timestamp\, or a string with a relative timestamp. See the general integration guide for more details 
    :param int validTo: When the filter stops being applied. Timestamp can be milliseconds since epoch\, ISO8601 timestamp\, or a string with a relative timestamp. See the general integration guide for more details 
    :param int index: The index of the filter. Filters with lower index will be ran first. \(default 5\)
    :param bool defaultAggregate: If true this filter will populate the created event with information from the selected event \(default false\)
    :param bool skipNewEvent: If true the filter will not submit triggered events for central storage \(default false\)
    :param bool enabled: Whether or not the filter is enabled \(default false\)
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationFailedException: on 412
    :raises ArgusException: on other status codes
    
    :returns: dictionary translated from JSON
    """

    route = "/eventfilters/v2/analysis".format()

    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    body = body or {}
    # Only send name if the argument was provided, dont send null values
    if name is not None:
        body.update({"name": name})
    # Only send index if the argument was provided, dont send null values
    if index is not None:
        body.update({"index": index})
    # Only send description if the argument was provided, dont send null values
    if description is not None:
        body.update({"description": description})
    # Only send customer if the argument was provided, dont send null values
    if customer is not None:
        body.update({"customer": customer})
    # Only send preStatementCode1 if the argument was provided, dont send null values
    if preStatementCode1 is not None:
        body.update({"preStatementCode1": preStatementCode1})
    # Only send preStatementCode2 if the argument was provided, dont send null values
    if preStatementCode2 is not None:
        body.update({"preStatementCode2": preStatementCode2})
    # Only send statementCode if the argument was provided, dont send null values
    if statementCode is not None:
        body.update({"statementCode": statementCode})
    # Only send triggerCode if the argument was provided, dont send null values
    if triggerCode is not None:
        body.update({"triggerCode": triggerCode})
    # Only send actionName if the argument was provided, dont send null values
    if actionName is not None:
        body.update({"actionName": actionName})
    # Only send defaultAggregate if the argument was provided, dont send null values
    if defaultAggregate is not None:
        body.update({"defaultAggregate": defaultAggregate})
    # Only send skipNewEvent if the argument was provided, dont send null values
    if skipNewEvent is not None:
        body.update({"skipNewEvent": skipNewEvent})
    # Only send labels if the argument was provided, dont send null values
    if labels is not None:
        body.update({"labels": labels})
    # Only send enabled if the argument was provided, dont send null values
    if enabled is not None:
        body.update({"enabled": enabled})
    # Only send comment if the argument was provided, dont send null values
    if comment is not None:
        body.update({"comment": comment})
    # Only send validFrom if the argument was provided, dont send null values
    if validFrom is not None:
        body.update({"validFrom": validFrom})
    # Only send validTo if the argument was provided, dont send null values
    if validTo is not None:
        body.update({"validTo": validTo})

    query_parameters = {}
    log.debug("POST %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.post(
        route,
        params=query_parameters or None,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response

@register_command(
    extending=("eventfilters", "v2", "analysis"),
    module=argus_cli_module
)
def delete_analysis_filter(
    id: int,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    """Deletes an analysis filter (DEV)
    
    :param int id: ID of filter to delete
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ArgusException: on other status codes
    
    :returns: dictionary translated from JSON
    """

    route = "/eventfilters/v2/analysis/{id}".format(id=id)

    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    body = body or {}

    query_parameters = {}
    log.debug("DELETE %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.delete(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response

@register_command(
    extending=("eventfilters", "v2", "analysis"),
    module=argus_cli_module
)
def get_analysis_filter(
    id: int,
    includeCode: bool = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    """Gets an analysis filter (DEV)
    
    :param int id: ID of filter
    :param bool includeCode: Whether to include code in AnnotationFilter
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ArgusException: on other status codes
    
    :returns: dictionary translated from JSON
    """

    route = "/eventfilters/v2/analysis/{id}".format(id=id,
        includeCode=includeCode)

    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    body = body or {}

    query_parameters = {}
    # Only send includeCode if the argument was provided, dont send null values
    if includeCode is not None:
        query_parameters.update({"includeCode": includeCode})
    
    log.debug("GET %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.get(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response

@register_command(
    extending=("eventfilters", "v2", "analysis"),
    module=argus_cli_module
)
def list_analysis_filter(
    keywords: str = None,
    keywordField: str = None,
    timestampField: str = None,
    sortBy: str = None,
    keywordMatch: str = "all",
    timestampMatch: str = "all",
    limit: int = 25,
    start: int = None,
    end: int = None,
    includeCode: bool = None,
    offset: int = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    """Simple search for analysis filters (DEV)
    
    :param list keywords: Search by keywords
    :param list keywordField: Set field strategy for keyword search
    :param list timestampField: Set field strategy for timestamp filtering
    :param list sortBy: Field to sort by
    :param str keywordMatch: Set match strategy for keyword search
    :param str timestampMatch: Set match strategy for timestamp filtering
    :param int limit: Limit result
    :param int start: Lower bound timestamp filter value
    :param int end: Upper bound timestamp filter value
    :param bool includeCode: Whether to include code in results
    :param int offset: Offset result
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationFailedException: on 412
    :raises ArgusException: on other status codes
    
    :returns: dictionary translated from JSON
    """

    route = "/eventfilters/v2/analysis".format(keywordMatch=keywordMatch,
        timestampMatch=timestampMatch,
        limit=limit,
        keywords=keywords,
        keywordField=keywordField,
        timestampField=timestampField,
        start=start,
        end=end,
        includeCode=includeCode,
        sortBy=sortBy,
        offset=offset)

    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    body = body or {}

    query_parameters = {}
    # Only send keywordMatch if the argument was provided, dont send null values
    if keywordMatch is not None:
        query_parameters.update({"keywordMatch": keywordMatch})
    
    # Only send timestampMatch if the argument was provided, dont send null values
    if timestampMatch is not None:
        query_parameters.update({"timestampMatch": timestampMatch})
    
    # Only send limit if the argument was provided, dont send null values
    if limit is not None:
        query_parameters.update({"limit": limit})
    
    # Only send keywords if the argument was provided, dont send null values
    if keywords is not None:
        query_parameters.update({"keywords": keywords})
    
    # Only send keywordField if the argument was provided, dont send null values
    if keywordField is not None:
        query_parameters.update({"keywordField": keywordField})
    
    # Only send timestampField if the argument was provided, dont send null values
    if timestampField is not None:
        query_parameters.update({"timestampField": timestampField})
    
    # Only send start if the argument was provided, dont send null values
    if start is not None:
        query_parameters.update({"start": start})
    
    # Only send end if the argument was provided, dont send null values
    if end is not None:
        query_parameters.update({"end": end})
    
    # Only send includeCode if the argument was provided, dont send null values
    if includeCode is not None:
        query_parameters.update({"includeCode": includeCode})
    
    # Only send sortBy if the argument was provided, dont send null values
    if sortBy is not None:
        query_parameters.update({"sortBy": sortBy})
    
    # Only send offset if the argument was provided, dont send null values
    if offset is not None:
        query_parameters.update({"offset": offset})
    
    log.debug("GET %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.get(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response

@register_command(
    extending=("eventfilters", "v2", "analysis"),
    module=argus_cli_module
)
def search_analysis_filter(
    order: str = None,
    sortBy: str = None,
    keywords: str = None,
    keywordFieldStrategy: str = None,
    keywordMatchStrategy: str = None,
    filterID: int = None,
    customer: str = None,
    user: str = None,
    userFieldStrategy: str = None,
    userMatchStrategy: str = None,
    timeFieldStrategy: str = None,
    timeMatchStrategy: str = None,
    includeFlags: str = None,
    excludeFlags: str = None,
    subCriteria: dict = None,
    startTimestamp: int = None,
    endTimestamp: int = None,
    includeCode: bool = None,
    limit: int = 25,
    offset: int = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    """Searches for analysis filters (DEV)
    
    :param list order: 
    :param list sortBy: 
    :param list keywords: Search for keywords against fields defined by keywordFieldStrategy 
    :param list keywordFieldStrategy: Defines which fields will be searched by keywords \(default all supported fields\) \(default all\)
    :param str keywordMatchStrategy: Defines how strict different keywords should be matched \(default match all keywords\) \(default all\)
    :param list filterID: Limit search to filters with the given IDs 
    :param list customer: Limit search to filters with the given customers\, identified by id or shortname 
    :param list user: Search using users identified by id or shortname. If given a group\, we will resolve all members of the group recursively 
    :param list userFieldStrategy: Defines which fields will be searched by user \(default all supported fields\) \(default all\)
    :param str userMatchStrategy: Defines how strict different keywords should be matched \(default match all keywords\) \(default all\)
    :param list timeFieldStrategy: Defines which timestamps will be included in the search \(default lastUpdated\) \(default lastUpdated\)
    :param str timeMatchStrategy: Defines how strict to match against different timestamps \(all\/any\) using start and end timestamp \(default any\) \(default any\)
    :param list includeFlags: Flags to include 
    :param list excludeFlags: Flags to exclude 
    :param list subCriteria: Subcriteria to add to the search 
    :param int startTimestamp: Restrict to a time frame based on the set timeFieldStrategy \(start timestamp\) \(default 0\)
    :param int endTimestamp: Restrict to a time frame based on the set timeFieldStrategy \(end timestamp\) \(default 0\)
    :param bool includeCode: Whether or not to include code in the result object. The code is only returned if set to true \(default false\)
    :param int limit: Limit maximum amount of results \(default 25\)
    :param int offset: Skip specified amount of results \(default 0\)
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationFailedException: on 412
    :raises ArgusException: on other status codes
    
    :returns: dictionary translated from JSON
    """

    route = "/eventfilters/v2/analysis/search".format()

    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    body = body or {}
    # Only send order if the argument was provided, dont send null values
    if order is not None:
        body.update({"order": order})
    # Only send sortBy if the argument was provided, dont send null values
    if sortBy is not None:
        body.update({"sortBy": sortBy})
    # Only send keywords if the argument was provided, dont send null values
    if keywords is not None:
        body.update({"keywords": keywords})
    # Only send keywordFieldStrategy if the argument was provided, dont send null values
    if keywordFieldStrategy is not None:
        body.update({"keywordFieldStrategy": keywordFieldStrategy})
    # Only send keywordMatchStrategy if the argument was provided, dont send null values
    if keywordMatchStrategy is not None:
        body.update({"keywordMatchStrategy": keywordMatchStrategy})
    # Only send filterID if the argument was provided, dont send null values
    if filterID is not None:
        body.update({"filterID": filterID})
    # Only send customer if the argument was provided, dont send null values
    if customer is not None:
        body.update({"customer": customer})
    # Only send user if the argument was provided, dont send null values
    if user is not None:
        body.update({"user": user})
    # Only send userFieldStrategy if the argument was provided, dont send null values
    if userFieldStrategy is not None:
        body.update({"userFieldStrategy": userFieldStrategy})
    # Only send userMatchStrategy if the argument was provided, dont send null values
    if userMatchStrategy is not None:
        body.update({"userMatchStrategy": userMatchStrategy})
    # Only send startTimestamp if the argument was provided, dont send null values
    if startTimestamp is not None:
        body.update({"startTimestamp": startTimestamp})
    # Only send endTimestamp if the argument was provided, dont send null values
    if endTimestamp is not None:
        body.update({"endTimestamp": endTimestamp})
    # Only send timeFieldStrategy if the argument was provided, dont send null values
    if timeFieldStrategy is not None:
        body.update({"timeFieldStrategy": timeFieldStrategy})
    # Only send timeMatchStrategy if the argument was provided, dont send null values
    if timeMatchStrategy is not None:
        body.update({"timeMatchStrategy": timeMatchStrategy})
    # Only send includeFlags if the argument was provided, dont send null values
    if includeFlags is not None:
        body.update({"includeFlags": includeFlags})
    # Only send excludeFlags if the argument was provided, dont send null values
    if excludeFlags is not None:
        body.update({"excludeFlags": excludeFlags})
    # Only send subCriteria if the argument was provided, dont send null values
    if subCriteria is not None:
        body.update({"subCriteria": subCriteria})
    # Only send includeCode if the argument was provided, dont send null values
    if includeCode is not None:
        body.update({"includeCode": includeCode})
    # Only send limit if the argument was provided, dont send null values
    if limit is not None:
        body.update({"limit": limit})
    # Only send offset if the argument was provided, dont send null values
    if offset is not None:
        body.update({"offset": offset})

    query_parameters = {}
    log.debug("POST %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.post(
        route,
        params=query_parameters or None,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response

@register_command(
    extending=("eventfilters", "v2", "analysis"),
    module=argus_cli_module
)
def update_analysis_filter(
    id: int,
    enabled: bool = None,
    name: str = None,
    index: int = None,
    validFrom: int = None,
    validTo: int = None,
    description: str = None,
    preStatementCode1: str = None,
    preStatementCode2: str = None,
    statementCode: str = None,
    triggerCode: str = None,
    actionName: str = None,
    defaultAggregate: bool = None,
    skipNewEvent: bool = None,
    removeLabels: str = None,
    addLabels: str = None,
    setLabels: str = None,
    comment: str = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    """Updates an analysis filter (DEV)
    
    :param int id: ID of filter
    :param bool enabled: 
    :param str name: The name of the filter 
    :param int index: The index of the filter. Filters with lower index will be ran first. 
    :param int validFrom: When the filter starts being applied. Timestamp can be milliseconds since epoch\, ISO8601 timestamp\, or a string with a relative timestamp. See the general integration guide for more details 
    :param int validTo: When the filter stops being applied. Timestamp can be milliseconds since epoch\, ISO8601 timestamp\, or a string with a relative timestamp. See the general integration guide for more details 
    :param str description: A description of the filter 
    :param str preStatementCode1: Optional esper code which does a pre selection before the statement code is run 
    :param str preStatementCode2: Optional esper code to do further pre selection on the result of preStatementCode1\, and applied before the statementCode 
    :param str statementCode: Esper code to select which events the filter will be applied to 
    :param str triggerCode: Groovy code which is run on the selected events 
    :param str actionName: The name of the signature which should be attached to the created event 
    :param bool defaultAggregate: If true this filter will populate the created event with information from the selected event 
    :param bool skipNewEvent: If true the filter will not submit triggered events for central storage 
    :param list removeLabels: A set of labels to remove from the filter.  Will be applied before adding labels and ignore filters that are not present. Will throw an exception if used in conjunction with setLabels  
    :param list addLabels: A set of labels to add to the filter.  Will be applied after removing labels. Will throw an exception if used in conjunction with setLabels  
    :param list setLabels: Replace the current set of labels with these. Will throw an exception if used in conjunction with addLabels or removeLabels 
    :param str comment: A comment describing why the filter was created 
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationFailedException: on 412
    :raises ArgusException: on other status codes
    
    :returns: dictionary translated from JSON
    """

    route = "/eventfilters/v2/analysis/{id}".format(id=id)

    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    body = body or {}
    # Only send enabled if the argument was provided, dont send null values
    if enabled is not None:
        body.update({"enabled": enabled})
    # Only send name if the argument was provided, dont send null values
    if name is not None:
        body.update({"name": name})
    # Only send index if the argument was provided, dont send null values
    if index is not None:
        body.update({"index": index})
    # Only send validFrom if the argument was provided, dont send null values
    if validFrom is not None:
        body.update({"validFrom": validFrom})
    # Only send validTo if the argument was provided, dont send null values
    if validTo is not None:
        body.update({"validTo": validTo})
    # Only send description if the argument was provided, dont send null values
    if description is not None:
        body.update({"description": description})
    # Only send preStatementCode1 if the argument was provided, dont send null values
    if preStatementCode1 is not None:
        body.update({"preStatementCode1": preStatementCode1})
    # Only send preStatementCode2 if the argument was provided, dont send null values
    if preStatementCode2 is not None:
        body.update({"preStatementCode2": preStatementCode2})
    # Only send statementCode if the argument was provided, dont send null values
    if statementCode is not None:
        body.update({"statementCode": statementCode})
    # Only send triggerCode if the argument was provided, dont send null values
    if triggerCode is not None:
        body.update({"triggerCode": triggerCode})
    # Only send actionName if the argument was provided, dont send null values
    if actionName is not None:
        body.update({"actionName": actionName})
    # Only send defaultAggregate if the argument was provided, dont send null values
    if defaultAggregate is not None:
        body.update({"defaultAggregate": defaultAggregate})
    # Only send skipNewEvent if the argument was provided, dont send null values
    if skipNewEvent is not None:
        body.update({"skipNewEvent": skipNewEvent})
    # Only send removeLabels if the argument was provided, dont send null values
    if removeLabels is not None:
        body.update({"removeLabels": removeLabels})
    # Only send addLabels if the argument was provided, dont send null values
    if addLabels is not None:
        body.update({"addLabels": addLabels})
    # Only send setLabels if the argument was provided, dont send null values
    if setLabels is not None:
        body.update({"setLabels": setLabels})
    # Only send comment if the argument was provided, dont send null values
    if comment is not None:
        body.update({"comment": comment})

    query_parameters = {}
    log.debug("PUT %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.put(
        route,
        params=query_parameters or None,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response
